from django.shortcuts import render, redirect  
from discussion.forms import  ThreadForm, TopicForm, SignupForm, SubjectForm, LoginForm
from discussion.models import  Subject_Student, Person, Subject, Topic, Thread, Person, Like

def login(request):
	invalid_user = False
	if request.method == "POST":
		form = LoginForm(request.POST)
		
		if form.is_valid():
			try:
				form_pid = form.cleaned_data.get("pid")
				form_pswd = form.cleaned_data.get("password")
				# print (form_pid)
				login = Person.objects.get(pid=form_pid, password=form_pswd)
				# print (login)
				# topic.subject = subject

				# request.session['session_context']['fname'] = login.fname
				# request.session['session_context']['lname'] = login.lname
				request.session['session_context'] = login.pid

				if(login.is_prof == True):
					return redirect('/prof')
					

				return redirect('/')
			except:
				invalid_user = True
		else:

			invalid_user = True
	else:

		form = LoginForm()
	# return render(request,'signup.html',{'form':form})
	
	return render(request, 'login.html', {'form':form, 'invalid_user':invalid_user})

def logout(request):
	try:
		del request.session['session_context']
	except:
		pass
	return render(request, 'default.html')

def signup(request):
	if request.method == "POST":
		form = SignupForm(request.POST)
		
		if form.is_valid():
			try:
				signup = form.save(commit=False)
				# topic.subject = subject
				form.save()	

				return redirect('/login')
			except:
				pass
	else:
		form = SignupForm()
	return render(request,'signup.html',{'form':form})


def default(request):
	is_prof = False
	try:
		context  = request.session['session_context']
	except:
		context = None

	return render(request, 'default.html' ,{'context':context})



def prof_home(request):
	try:
		context  = request.session['session_context']
	except:
		context = None

	return render(request, 'prof_home.html' ,{'context':context})

def prof_subjects(request):
	context  = request.session['session_context']
	try:
		prof = Person.objects.get(pid=context)
		subjects = Subject.objects.filter(prof = prof)
	except:
		subjects = None

	if request.method == "POST":
		form = SubjectForm(request.POST)
		
		if form.is_valid():
			try:
				subject = form.save(commit=False)
				subject.prof = prof
				form.save()	

				return redirect('/prof/subjects')
			except Exception as e:
				print (e)
	else:
		form = SubjectForm()
	
	
	return render(request,"prof_subjects.html",{'subjects':subjects, 'context':context, 'form':form})

def student_list(request, sub_code):
	context  = request.session['session_context']
	sub_studs = None
	try:
		prof = Person.objects.get(pid=context)
		subject = Subject.objects.get(prof = prof, sub_code=sub_code)
		sub_studs = Subject_Student.objects.filter(subject = subject)
	except:
		sub_studs = None
	
	
	return render(request,"students_list.html",{'subject':subject, 'context':context, 'sub_studs':sub_studs})

def accept_student(request, sub_code, pid):
	context  = request.session['session_context']
	try:	

		sub_stud_to_accept = Subject_Student.objects.get(student = Person.objects.get(pid=pid), subject = Subject.objects.get(sub_code=sub_code))
		sub_stud_to_accept.is_accepted = True
		sub_stud_to_accept.save()

	except Exception as e:
		print (e)
	
	return redirect('/prof/student_list/'+sub_code)

def request_subject(request, sub_code):
	context  = request.session['session_context']
	try:	
		subject = Subject.objects.get(sub_code = sub_code)
		student = Person.objects.get(pid = context)

		sub_stud = Subject_Student(subject = subject, student = student)
		sub_stud.save()

	except Exception as e:
		print (e)
	
	return redirect('/available_subjects')

def reject_student(request, sub_code, pid):
	context  = request.session['session_context']
	try:
		Subject_Student.objects.get(student = Person.objects.get(pid=pid), subject = Subject.objects.get(sub_code=sub_code)).delete()
	except Exception as e:
		print (e)	
	return redirect('/prof/student_list/'+sub_code)

def reject_subject(request, sub_code):
	context  = request.session['session_context']
	try:
		Subject_Student.objects.get(student = Person.objects.get(pid=context), subject = Subject.objects.get(sub_code=sub_code)).delete()
	except Exception as e:
		print (e)	
	return redirect('/pending_subjects')

def withdraw_subject(request, sub_code):
	context  = request.session['session_context']
	try:
		Subject_Student.objects.get(student = Person.objects.get(pid=context), subject = Subject.objects.get(sub_code=sub_code)).delete()
	except Exception as e:
		print (e)	
	return redirect('/subjects')



def show_subjects(request):
	context  = request.session['session_context']
	try:
		person = Person.objects.get(pid=context)
	except:
		return redirect('/')
	subject_student = Subject_Student.objects.filter(student = person)
	return render(request,"subjects.html",{'subject_student':subject_student, 'context':context})

def show_all_subjects(request):
	context  = request.session['session_context']
	subjects = Subject.objects.all()

	sub_studs = Subject_Student.objects.filter(student = Person.objects.get(pid=context))

	my_subs = []

	for sub_stud in sub_studs:
		my_subs.append(sub_stud.subject.sub_code)

	subjects = subjects.exclude(sub_code__in=my_subs)
	
	return render(request,"available_subjects.html",{'subjects':subjects, 'context':context})

def show_pending_subjects(request):
	context  = request.session['session_context']

	sub_studs = Subject_Student.objects.filter(student = Person.objects.get(pid=context))
	
	return render(request,"pending_subjects.html",{'sub_studs':sub_studs, 'context':context})

def show_topics(request, id):
	context  = request.session['session_context']
	is_prof = False
	try:
		subject = Subject.objects.get(sub_code=id)
	except:
		return redirect('/')

	try:
		person = Person.objects.get(pid=context)
		is_prof = person.is_prof
	except Exception as e:
		print (e)

	if is_prof:
		topics = Topic.objects.filter(subject = subject, is_prof_included=True)
	else:
		topics = Topic.objects.filter(subject = subject)

	if request.method == "POST":
		form = TopicForm(request.POST)
		
		if form.is_valid():
			try:
				topic = form.save(commit=False)
				topic.subject = subject
				if is_prof:
					topic.is_prof_included = True
				form.save()	

				return redirect('/topics/' + str(id))
			except:
				pass
	else:
		form = TopicForm()

	if is_prof:
		return render(request,'prof_discussion_topics.html',{'form':form, 'sub_code':id, 'topics':topics, 'context':context, 'is_prof':is_prof})
	else:
		return render(request,'discussion_topics.html',{'form':form, 'sub_code':id, 'topics':topics, 'context':context, 'is_prof':is_prof})

def show_threads(request, id):
	context  = request.session['session_context']
	is_prof = False
	try:
		person = Person.objects.get(pid=context)
		is_prof = person.is_prof
	except Exception as e:
		print (e)
	try:
		topic = Topic.objects.get(id=id)
	except:
		return redirect('/')
	threads = Thread.objects.filter(topic = topic)
	if request.method == "POST":
		person = Person.objects.get(pid=context)
		form = ThreadForm(request.POST)
		
		if form.is_valid():
			try:
				thread = form.save(commit=False)
				thread.topic = topic
				thread.person = person
				form.save()

				return redirect('/threads/' + str(id))
			except:
				pass
	else:
		form = ThreadForm()

	if is_prof:
		return render(request,'prof_discussion_threads.html',{'form':form, 'topic_id':id, 'threads':threads, 'context':context, 'is_prof':is_prof}) 
	else:
		return render(request,'discussion_thread.html',{'form':form, 'topic_id':id, 'threads':threads, 'context':context, 'is_prof':is_prof}) 

def update_thread_like_count(request, thread_id, topic_id):
	context  = request.session['session_context']
	try:
		thread = Thread.objects.get(id=thread_id)
		person = Person.objects.get(pid=context)
	except Exception as e:
		print (e)
		return redirect('/threads/'+str(topic_id))

	try:
		like = Like.objects.get(thread=thread, person=person)
		if like.is_liked == False :
			like.is_liked = True
			like.save()
			thread.likes = thread.likes + 1
			thread.dislikes = thread.dislikes - 1
			thread.save()
		else:
			pass
	except:
		like = Like(thread=thread, person=person, is_liked = True)
		like.save()
		thread.likes = thread.likes + 1
		thread.save()


	return redirect('/threads/'+str(topic_id))



def update_thread_dislike_count(request, thread_id, topic_id):
	context  = request.session['session_context']
	try:
		thread = Thread.objects.get(id=thread_id)
		person = Person.objects.get(pid=context)
	except Exception as e:
		print (e)
		return redirect('/threads/'+str(topic_id))

	try:
		like = Like.objects.get(thread=thread, person=person)
		if like.is_liked == True :
			like.is_liked = False
			like.save()
			thread.likes = thread.likes - 1
			thread.dislikes = thread.dislikes + 1
			thread.save()
		else:
			pass
	except:
		like = Like(thread=thread, person=person, is_liked = False)
		like.save()
		thread.dislikes = thread.dislikes + 1
		thread.save()


	return redirect('/threads/'+str(topic_id))
